package temps;

/**
 * La classe Date décrit des jours du calendrier.
 *
 * @author Philippe
 * @version 1
 */
public class Date {
    private int leJour, leMois, lAnnee;

    /**
     * Constructeur d'une Date
     */
    public Date() {
        leJour = 1;
        leMois = 1;
        lAnnee = 2025;
    }
    
    /**
     * Constructeur d'une Date
     * 
     * @param  j un entier qui précise le jour du mois
     * @param  m un entier qui précise le mois de la date
     * @param  a un entier qui précise l'année de la date
     */
    public Date(int j, int m, int a) {
        leJour = j;
        leMois = m;
        lAnnee = a;
    }

    /**
     * renvoie le jour du mois de la date
     *  
     * @return un entier qui précise le jour du mois
     */
    public int jour() {
        return leJour;
    }
    
    /**
     * renvoie le mois de la date
     * 
     * @return un entier qui précise le mois de la date
     */
    public int mois() {
    	return leMois;
    }
    
    /**
     * renvoie l'année de la date
     * 
     * @return un entier qui précise l'année de la date
     */
    public int annee() {
    	return lAnnee;
    }
    
    /**
     * méthode qui renvoie le nombre de jour d'un mois quelconque
     * 
     * @param mois un entier précisant le mois de l'année
     * @param annee un entier précisant l'année
     */
    public static int nombreDeJoursDuMois(int mois, int annee) {
        switch (mois) {
            case 1 :
            case 3 :
            case 5 :
            case 7 :
            case 8 :
            case 10:
            case 12:
                return 31;
            case 4 :
            case 6 :
            case 9 :
            case 11:
                return 30;
            case 2 :
                return bissextile(annee) ? 29 : 28;
            default:
                return 0;
        }
    }
    
    /**
     * détermine si une année est bissextile ou non
     * 
     * @param annee un entier précisant l'année
     * @return vrai si l'année compte 366 jours, faux si elle compte 365 jours
     */
    public static boolean bissextile(int annee) {
        if (annee % 4 != 0) return false;
        if (annee % 100 == 0) return annee % 400 == 0;
        return true;
    }

    /**
     * méthode qui fait passer la date au jour suivant
     */
    public void jourPlus1() {
        leJour++;
        if (leJour > nombreDeJoursDuMois(leMois, lAnnee)) {
            leJour = 1;
            moisPlus1();
        }
    }
    
    /**
     * méthode qui fait passer la date au mois suivant
     */
    public void moisPlus1() {
        leMois++;
        if (leMois > 12) {
            leMois = 1;
            lAnnee++;
        } else {
        	if (leMois == 2) {
        		int max = Date.nombreDeJoursDuMois(leMois, lAnnee);
        		if (leJour > max) leJour = max;
        	}
        }
    }

    /**
     * méthode qui fait passer la date à l'année suivante
     */
    public void anneePlus1() {
        lAnnee++;
        if (leMois == 2 && leJour == 29)
        	leJour = 28;
    }

    /**
     * méthode qui fait passer la date au jour précédant
     */
    public void jourMoins1() {
        leJour--;
        if (leJour < 1) {
            moisMoins1();
            leJour = nombreDeJoursDuMois(leMois, lAnnee);
        }
    }
    
    /**
     * méthode qui fait passer la date au mois précédant
     */
    public void moisMoins1() {
        leMois--;
        if (leMois < 1) {
            leMois = 12;
            anneeMoins1();
        } else {
        	int maxJours = nombreDeJoursDuMois(leMois, lAnnee);
        	if (leJour > maxJours) leJour = maxJours; 
        }
    }

    /**
     * méthode qui fait passer la date à l'année précédante
     */
    public void anneeMoins1() {
        lAnnee--;
        if (leMois == 2 && leJour == 29)
        	leJour = 28;
    }

    /**
     * renvoie le texte notant la date
     *
     * @return     le texte de la date au format jj/mm/aaaa
     */
    public String toString() {
        String txt;
        /* 
        if (j < 10) txt = "0";
        else txt = "";
        */
       txt = (leJour < 10) ? "0" : "";
       txt += leJour;
       txt += '/';
       if (leMois < 10) txt += "0";
       txt += leMois;
       txt += '/';
       txt += lAnnee;
       return txt;
    }
}

